//From: https://developer.mozilla.org/en-US/docs/WebGL/Getting_started_with_WebGL
function initWebGL(canvas) {
	// Initialize the global variable gl to null.
	gl = null;
   
	try {
		// Try to grab the standard context. If it fails, fallback to experimental.
		var attribs = {};
		attribs.antialias = true;
		//attribs.stencil = true;
		gl = canvas.getContext("webgl", attribs) || canvas.getContext("experimental-webgl", attribs);
	}
	catch(e) {}
   
	// If we don't have a GL context, give up now
	if (!gl) {
		alert("Unable to initialize WebGL. Your browser may not support it.");
	}
}

function getShader(gl, id) {
	var shaderScript, theSource, currentChild, shader;
	 
	shaderScript = document.getElementById(id);
	 
	if (!shaderScript) {
		return null;
	}
	 
	theSource = "";
	currentChild = shaderScript.firstChild;
	 
	while(currentChild) {
		if (currentChild.nodeType == currentChild.TEXT_NODE) {
			theSource += currentChild.textContent;
		}
	   
		currentChild = currentChild.nextSibling;
	}
	if (shaderScript.type == "x-shader/x-fragment") {
		shader = gl.createShader(gl.FRAGMENT_SHADER);
	} else if (shaderScript.type == "x-shader/x-vertex") {
		shader = gl.createShader(gl.VERTEX_SHADER);
	} else {
		// Unknown shader type
		return null;
	}
	gl.shaderSource(shader, theSource);
	   
	// Compile the shader program
	gl.compileShader(shader);  
	   
	// See if it compiled successfully
	if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {  
		alert("An error occurred compiling the shaders: " + gl.getShaderInfoLog(shader));  
		return null;  
	}
	   
	return shader;
}
function getProgram(fragId,vertId) {
	var fragmentShader = getShader(gl, fragId);
	var vertexShader = getShader(gl, vertId);
   
	// Create the shader program
   
	var program = gl.createProgram();
	gl.attachShader(program, vertexShader);
	gl.attachShader(program, fragmentShader);
	gl.linkProgram(program);
   
	// If creating the shader program failed, alert
	if (!gl.getProgramParameter(program, gl.LINK_STATUS)) {
		alert("Unable to initialize the shader program.");
		return null;
	}

	return program;
}

function textureFromImage(img, do_mip) {
 	var tex = gl.createTexture();
	gl.bindTexture(gl.TEXTURE_2D, tex);
	gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, img);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);
	if (do_mip) {
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR_MIPMAP_LINEAR);
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
		gl.generateMipmap(gl.TEXTURE_2D);
	} else {
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
	}
	gl.bindTexture(gl.TEXTURE_2D, null);
	return tex;
}


function glEmit(verts,colors,texCoords,mode) {
	var program = gl.getParameter(gl.CURRENT_PROGRAM);

	if (undefined == mode) {
		mode = gl.TRIANGLE_STRIP;
		//console.debug("mode not passed.");
	} 

	var aVertex = gl.getAttribLocation(program, "aVertex");
	var vertsBuffer = gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, vertsBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array( verts ), gl.STREAM_DRAW);
	gl.enableVertexAttribArray(aVertex);
	gl.vertexAttribPointer(aVertex, 2, gl.FLOAT, false, 0, 0);

	var aColor = -1;
	var colorsBuffer = null;
	if (colors) {
		aColor = gl.getAttribLocation(program, "aColor");
		if (colors.length == verts.length * 2) {
			var colorsBuffer = gl.createBuffer();
			gl.bindBuffer(gl.ARRAY_BUFFER, colorsBuffer);
			gl.bufferData(gl.ARRAY_BUFFER, new Float32Array( colors ), gl.STREAM_DRAW);
			gl.enableVertexAttribArray(aColor);
			gl.vertexAttribPointer(aColor, 4, gl.FLOAT, false, 0, 0);
		} else {
			gl.vertexAttrib4fv(aColor, colors);
		}
	}

	var aTexCoord = -1;
	var texCoordsBuffer = null;
	if (texCoords) {
		aTexCoord = gl.getAttribLocation(program, "aTexCoord");
		texCoordsBuffer = gl.createBuffer();
		gl.bindBuffer(gl.ARRAY_BUFFER, texCoordsBuffer);
		gl.bufferData(gl.ARRAY_BUFFER, new Float32Array( texCoords ), gl.STREAM_DRAW);
		gl.enableVertexAttribArray(aTexCoord);
		gl.vertexAttribPointer(aTexCoord, 2, gl.FLOAT, false, 0, 0);
	}

	gl.drawArrays(mode, 0, verts.length / 2);


	if (texCoordsBuffer) {
		gl.disableVertexAttribArray(aTexCoord);
		gl.deleteBuffer(texCoordsBuffer);
		texCoordsBuffer = null;
	}

	if (colorsBuffer) {
		gl.disableVertexAttribArray(aColor);
		gl.deleteBuffer(colorsBuffer);
		colorsBuffer = null;
	}

	gl.disableVertexAttribArray(aVertex);
	gl.deleteBuffer(vertsBuffer);
	vertsBuffer = null;
}
