/*
 * Rainbow game for LD25 compo.
 * webGL stuff based on examples in Mozilla docs:
 *   https://developer.mozilla.org/en-US/docs/WebGL
 * (c) 2012 Jim McCann
 */

//Loading stuff:
var pendingAssets = 0;
var totalAssets = 0;

var introTextures = [null, null, null];
var outroTextures = [null, null];
var peonTexture = null;
var peoneTexture = null;
var shimmerTexture = null;

var thunderAudio = null;
var glimmersAudio = null;
var glissAudio = null;
function playGliss() {
	glissAudio.pause();
	glissAudio.currentTime = 0.0;
	glissAudio.play();
}
var hitsAudio = [
	null,
	null,
	null,
	null,
	null,
];
var nextHit = 0;
function playHit() {
	hitsAudio[nextHit].pause();
	hitsAudio[nextHit].currentTime = 0.0;
	hitsAudio[nextHit].volume = 0.7 + Math.random() * 0.1;
	hitsAudio[nextHit].play();
	nextHit = (nextHit + 1) % hitsAudio.length;
}
function setMuted(muted) {
	thunderAudio.muted = muted;
	glimmersAudio.muted = muted;
	glissAudio.muted = muted;
	for (var i = 0; i < hitsAudio.length; ++i) {
		hitsAudio[i].muted = muted;
	}
}


//Interaction stuff:

var lastTimestamp;

var mouse = null;
var mousedown = false;

var camera = {
	at: {x:0, y:0, z:0},
	up: {x:10, y:0},
};

var board = null;
var bow = null;
var bowStart = {x:0, y:0};

var growRange = null;
var growing = false;
var growAcc = 0.0;

function clearBoard() {
	board = null;
	bow = null;
	bowStart = {x:0, y:0};
	growRange = null;
	growing = false;
	growAcc = 0.0;
}

//Game constants:

var BOW_WIDTH = 6;
var BOW_STEPS = 20;
var BOW_STRIPE_RADIUS = 0.4 * BOW_WIDTH / BOW_STEPS;


var STEER_WIDTH = BOW_WIDTH * 0.5;
var BOW_SPEED = 0.1;

var TICK = 1.0 / 60.0;

var STATE_LOADING = "loading";
var STATE_INTRO = "intro";
var STATE_LEVEL = "level";
var STATE_OUTRO = "outro";

//state switch stuff:

var levelIndex = 0;

var troStep = 0;

var state = STATE_LOADING;
var stateFade = 1.0;
var stateFadeOut = false;

//intro/outro stuff:
var crowd = [];
var CROWD_RAD = 100 / 600;

function setupCrowd() {
	crowd = [];
	var pos = [
		[478, 209],
		[365, 193],
		[560, 157],
		[405, 109],
		[512, 103],
		[559, 25],
		[416, 9],
		];
	var cols = [
		[  83,   0, 176, 0.5],
		[  83,   0, 176, 1.0],
		[  31,  11, 239, 1.0],
		[   0, 208, 157, 1.0],
		[   0, 183,   0, 1.0],
		[ 204, 210,   0, 1.0],
		[ 255,  59,   0, 1.0],
		[ 255,  59,   0, 0.5]
	];

	for (var i = 0; i < 7; ++i) {
		var cf = Math.random() * (cols.length-1);
		var ci = cf | 0;
		cf -= ci;
		var r = ((cols[ci+1][0] - cols[ci][0]) * cf + cols[ci][0]) / 255.0;
		var g = ((cols[ci+1][1] - cols[ci][1]) * cf + cols[ci][1]) / 255.0;
		var b = ((cols[ci+1][2] - cols[ci][2]) * cf + cols[ci][2]) / 255.0;
		var a = ((cols[ci+1][3] - cols[ci][3]) * cf + cols[ci][3]);
		crowd[i] = {
			x: (pos[i][0] + 50 - 350) / 300,
			y: (pos[i][1] - 300) / 300,
			tint:[r*a,g*a,b*a,1.0],
			acc: Math.random(),
			period: 1.0 + Math.random(),
		};
	}
}

//GL stuff:
var solidProgram = null;
var texProgram = null;

function localMouse() {
	if (!mouse) return null;
	if (!board) return null;
	var el = document.getElementById("game");
	var canvasWidth = el.width;
	var canvasHeight = el.height;

	var lx = 2.0 * (mouse.x - 0.5 * canvasWidth) / canvasHeight;
	var ly = 2.0 * (0.5 * canvasHeight - mouse.y) / canvasHeight;
	return {
		x:lx* camera.up.y + ly*camera.up.x + camera.at.x,
		y:lx*-camera.up.x + ly*camera.up.y + camera.at.y
	};
}

function setupBoard() {
	clearBoard();

	var proto = levels[levelIndex].proto;

	board = {
		width: proto[0].length,
		height: proto.length,
		tiles: [],
		peons: []
	};
	for (var boardRow = 0; boardRow < proto.length; ++boardRow) {
		var row = proto.length - 1 - boardRow;
		board.tiles[boardRow] = [];
		for (var col = 0; col < proto[row].length; ++col) {
			if (proto[row][col] == "x") {
				board.tiles[boardRow][col] =-1;
			} else if (proto[row][col] == "P") {
				board.peons.push({
					x:col,
					y:boardRow,
					enlightened:false,
					efade:0.0,
					tint:[1.0, 1.0, 1.0, 1.0]
				});
				board.tiles[boardRow][col] = board.peons.length;
			} else {
				board.tiles[boardRow][col] = 0;
			}
		}
	}

	var el = document.getElementById("game");
	var canvasWidth = el.width;
	var canvasHeight = el.height;

	var a = canvasWidth / canvasHeight;

	var s = Math.max(
		board.height + 10.0 + 5.0 + 5.0,
		(board.width + 5.0 + 5.0) / a);

	bowStart.x = board.width / 2.0;
	bowStart.y = board.height + 10.0;


	//setup camera to view board:

	camera.at.x = board.width / 2.0;
	camera.at.y = 0.5 * (s - 5.0 - 5.0);
	camera.up.x = 0.0;
	camera.up.y = 0.5 * s;

	//setup board buffer with static level geometry:

	var verts = [];

	{ //ground:
		var groundWidth = 10.0;
		var groundDepth = 6.0;
		verts.push(-groundWidth,-groundDepth);
		verts.push(board.width + groundWidth,-groundDepth);
		verts.push(board.width + groundWidth,0.0);

		verts.push(-groundWidth,-groundDepth);
		verts.push(board.width + groundWidth,0.0);
		verts.push(-groundWidth,0.0);
	}

	for (var row = 0; row < board.height; ++row) {
		for (var col = 0; col < board.width; ++col) {
			if (board.tiles[row][col] == -1) {
				verts.push(col,row);
				verts.push(col+1,row);
				verts.push(col+1,row+1);
				verts.push(col,row);
				verts.push(col+1,row+1);
				verts.push(col,row+1);
			}
		}
	}

	board.vertexBuffer = gl.createBuffer();
	board.vertexCount = verts.length / 2;
	gl.bindBuffer(gl.ARRAY_BUFFER, board.vertexBuffer);
	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array( verts ), gl.STATIC_DRAW);
	gl.bindBuffer(gl.ARRAY_BUFFER, null);

	var ow = document.getElementById("onward");
	ow.hidden = true;
}

function setupBow() {
	bow = [];
	var cols = [
		[  83,   0, 176, 0.5],
		[  83,   0, 176, 1.0],
		[  31,  11, 239, 1.0],
		[   0, 208, 157, 1.0],
		[   0, 183,   0, 1.0],
		[ 204, 210,   0, 1.0],
		[ 255,  59,   0, 1.0],
		[ 255,  59,   0, 0.5]
	];
	for (var i = 0; i < BOW_STEPS; ++i) {
		var amt = i / (BOW_STEPS - 1);
		var x = bowStart.x + (amt - 0.5) * BOW_WIDTH
		var cf = ((i + 0.5) / BOW_STEPS) * (cols.length - 1);
		var ci = cf | 0;
		cf -= ci;
		var r = ((cols[ci+1][0] - cols[ci][0]) * cf + cols[ci][0]) / 255.0;
		var g = ((cols[ci+1][1] - cols[ci][1]) * cf + cols[ci][1]) / 255.0;
		var b = ((cols[ci+1][2] - cols[ci][2]) * cf + cols[ci][2]) / 255.0;
		var a = ((cols[ci+1][3] - cols[ci][3]) * cf + cols[ci][3]);

		bow[i] = {
			x:[x, x],
			y:[bowStart.y+30, bowStart.y],
			style: [r * a,g * a,b * a, 1.0],
			active: true,
			shift: Math.random(),
			bias: Math.random(),
		};

	}
}

function addVert(bow,x,y) {
	bow.x[bow.x.length] = x;
	bow.y[bow.y.length] = y;
	//TODO: accumulate path verts so we don't have to keep recalcing.
}

function normalize(vec) {
	var len = vec.x * vec.x + vec.y * vec.y;
	if (len == 0.0) return { x: 1.0, y: 0.0 };
	len = Math.sqrt(len);
	return {
		x: vec.x / len,
		y: vec.y / len
	};
}

function setCameraUniforms() {

	var el = document.getElementById("game");
	var canvasWidth = el.width;
	var canvasHeight = el.height;

	var lenSq = (camera.up.x * camera.up.x + camera.up.y * camera.up.y);
	var invUp = {
		x: camera.up.x / lenSq,
		y: camera.up.y / lenSq
	};

	var a = canvasWidth / canvasHeight;

	var invRight = {
		x: camera.up.y / (lenSq * a),
		y:-camera.up.x / (lenSq * a)
	};
	var PMatrix = [
		invRight.x, invUp.x, 0.0, 0.0,
		invRight.y, invUp.y, 0.0, 0.0,
		0.0, 0.0, 1.0, 0.0,
		invRight.x * -camera.at.x + invRight.y * -camera.at.y,
			invUp.x * -camera.at.x + invUp.y * -camera.at.y,
			0.0, 1.0
	];
	gl.uniformMatrix4fv(gl.getUniformLocation(gl.getParameter(gl.CURRENT_PROGRAM), "uPMatrix"), false, PMatrix);
}


function setScreenUniforms() {

	var el = document.getElementById("game");
	var canvasWidth = el.width;
	var canvasHeight = el.height;

	var a = canvasWidth / canvasHeight;

	var PMatrix = [
		1.0 / a, 0.0, 0.0, 0.0,
		0.0, 1.0, 0.0, 0.0,
		0.0, 0.0, 1.0, 0.0,
		0.0, 0.0, 0.0, 1.0
	];
	gl.uniformMatrix4fv(gl.getUniformLocation(gl.getParameter(gl.CURRENT_PROGRAM), "uPMatrix"), false, PMatrix);
}

function growStep(lm, tRange, tx,ty,sP, tfade,verts,colors,sVerts,sColors,sTexCoords, commit) {
	var ai = tRange[0];
	var bi = tRange[1];

	if (ai < 0 || ai >= tx.length) {
		console.debug("Bad range: " + tRange);
	}

	var norm;
	var a = {
		x:tx[ai],
		y:ty[ai]
	};
	var b = {
		x:tx[bi],
		y:ty[bi]
	};
	if (ai == bi) {
		norm = normalize({
			x:b.x[b.x.length-1] - b.x[b.x.length-2],
			y:b.y[b.y.length-1] - b.y[b.y.length-2]
		});
	} else {
		var perp = {
			x:a.x[a.x.length-1] - b.x[b.x.length-1],
			y:a.y[a.y.length-1] - b.y[b.y.length-1]
		};
		norm = normalize({x:-perp.y,y:perp.x});
	}
	var perp = {x:norm.y, y:-norm.x};
	var center = {
		x:0.5*(a.x[a.x.length-1] + b.x[b.x.length-1]),
		y:0.5*(a.y[a.y.length-1] + b.y[b.y.length-1])
	};
	var da =
		(a.x[a.x.length-1] - center.x) * perp.x
		+ (a.y[a.y.length-1] - center.y) * perp.y;
	var db =
		(b.x[b.x.length-1] - center.x) * perp.x
		+ (b.y[b.y.length-1] - center.y) * perp.y;


	da += 0.5 * BOW_WIDTH / BOW_STEPS;
	db -= 0.5 * BOW_WIDTH / BOW_STEPS;


	var width = Math.max(Math.abs(da), Math.abs(db));

	var t_center = BOW_SPEED;

	var max_ang = Math.PI / 4.0;
	if (width >= BOW_SPEED) {
		var test = Math.asin(BOW_SPEED / width) * 0.9;
		if (test < max_ang) {
			max_ang = test;
		}
	}

	var dm = 
		(lm.x - center.x) * perp.x
		+ (lm.y - center.y) * perp.y;

	if (dm < -STEER_WIDTH) dm =-STEER_WIDTH;
	if (dm > STEER_WIDTH) dm = STEER_WIDTH;
	var ang =-max_ang * (dm / STEER_WIDTH);
			
/*
	var ang =-Math.atan2(
		(lm.x - center.x) * perp.x + (lm.y - center.y) * perp.y,
		(lm.x - center.x) * norm.x + (lm.y - center.y) * norm.y
	);
	if (ang > max_ang) ang = max_ang;
	if (ang <-max_ang) ang =-max_ang;
*/

	var xf_norm = {
		x:Math.cos(ang),
		y:Math.sin(ang)
	};

	var active = [];

	//Advance growing bows:
	for (var i = ai; i <= bi; ++i) {
		var bw = bow[i];
		var b = {
			x:tx[i],
			y:ty[i]
		};
		var x = b.x[b.x.length-1];
		var y = b.y[b.y.length-1];

		var d_norm = normalize({
			x:x - b.x[b.x.length-2],
			y:y - b.y[b.y.length-2],
		});
		var d_perp = {
			x:BOW_STRIPE_RADIUS * d_norm.y,
			y:BOW_STRIPE_RADIUS *-d_norm.x,
		};

		var fade;
		if (tfade < 0.75) {
			fade = 0.5 * (1.0 - tfade / 0.75);
		} else {
			fade = 0.0;
		}
				
		if (verts.length) {
			verts.push(verts[verts.length-2], verts[verts.length-1]);
			verts.push(x-d_perp.x,y-d_perp.y);
			colors.push(colors[colors.length-4], colors[colors.length-3], colors[colors.length-2], colors[colors.length-1]);
			colors.push(bw.style[0], bw.style[1], bw.style[2], bw.style[3] * fade);
		}
		verts.push(x-d_perp.x,y-d_perp.y);
		verts.push(x+d_perp.x,y+d_perp.y);
		colors.push(bw.style[0], bw.style[1], bw.style[2], bw.style[3] * fade);
		colors.push(bw.style[0], bw.style[1], bw.style[2], bw.style[3] * fade);

		var w = 0.1
		var x0 = bw.bias;
		var x1 = bw.bias + w;
		var p = sP[i];

		var sFade;
		if (tfade < 0.5) {
			sFade = tfade + 0.5;
		} else {
			sFade = (1.0 - tfade) * 2.0;
		}

		if (sVerts.length) {
			sVerts.push(sVerts[sVerts.length-2], sVerts[sVerts.length-1]);
			sVerts.push(x-d_perp.x,y-d_perp.y);
			sColors.push(sColors[sColors.length-4], sColors[sColors.length-3], sColors[sColors.length-2], sColors[sColors.length-1]);
			sColors.push(1.0, 1.0, 1.0, sFade);
			sTexCoords.push(sTexCoords[sTexCoords.length-2], sTexCoords[sTexCoords.length-1]);
			sTexCoords.push(x0,p);
		}
		sVerts.push(x-d_perp.x,y-d_perp.y);
		sVerts.push(x+d_perp.x,y+d_perp.y);
		sColors.push(1.0, 1.0, 1.0, sFade);
		sColors.push(1.0, 1.0, 1.0, sFade);
		sTexCoords.push(x0,p);
		sTexCoords.push(x1,p);


		var prev_center = center;
		var prev_norm = norm;
		var next_norm = {
			x:prev_norm.x * xf_norm.x +-prev_norm.y * xf_norm.y,
			y:prev_norm.y * xf_norm.x + prev_norm.x * xf_norm.y
		};
		var next_center = {
			x:prev_center.x + t_center * next_norm.x,
			y:prev_center.y + t_center * next_norm.y
		};
		var px = x;
		var py = y;

		x = x - prev_center.x;
		y = y - prev_center.y;
		var lx = x *-prev_norm.y + y * prev_norm.x;
		var ly = x * prev_norm.x + y * prev_norm.y;

		x = lx *-next_norm.y + ly * next_norm.x + next_center.x;
		y = lx * next_norm.x + ly * next_norm.y + next_center.y;


		var d_norm = normalize({
			x:x - px,
			y:y - py,
		});
		var d_perp = {
			x:BOW_STRIPE_RADIUS * d_norm.y,
			y:BOW_STRIPE_RADIUS *-d_norm.x,
		};

		verts.push(px-d_perp.x,py-d_perp.y);
		verts.push(px+d_perp.x,py+d_perp.y);
		verts.push(x-d_perp.x,y-d_perp.y);
		verts.push(x+d_perp.x,y+d_perp.y);

		colors.push(bw.style[0], bw.style[1], bw.style[2], bw.style[3] * fade);
		colors.push(bw.style[0], bw.style[1], bw.style[2], bw.style[3] * fade);
		colors.push(bw.style[0], bw.style[1], bw.style[2], bw.style[3] * fade);
		colors.push(bw.style[0], bw.style[1], bw.style[2], bw.style[3] * fade);

		sVerts.push(px-d_perp.x,py-d_perp.y);
		sVerts.push(px+d_perp.x,py+d_perp.y);
		sVerts.push(x-d_perp.x,y-d_perp.y);
		sVerts.push(x+d_perp.x,y+d_perp.y);
		sColors.push(1.0,1.0,1.0,sFade);
		sColors.push(1.0,1.0,1.0,sFade);
		sColors.push(1.0,1.0,1.0,sFade);
		sColors.push(1.0,1.0,1.0,sFade);
		sTexCoords.push(x0,p);
		sTexCoords.push(x1,p);
		p += Math.sqrt((px-x)*(px-x)+(py-y)*(py-y)) / (2.0 * BOW_STRIPE_RADIUS) * 0.25 * w;
		sTexCoords.push(x0,p);
		sTexCoords.push(x1,p);
		sP[i] = p;

		var collided = false;
		var collidedPeon = null;
		if (y < 0.0) collided = true;
		if (x > 0 && y > 0) {
			var ix = x | 0;
			var iy = y | 0;
			if (ix < board.width && iy < board.height) {
				if (board.tiles[iy][ix] == -1) {
					collided = true;
				} else if (board.tiles[iy][ix] > 0) {
					collidedPeon = board.peons[board.tiles[iy][ix] - 1];
					if (collidedPeon.enlightened) {
						collidedPeon = null;
					} else {
						collided = true;
					}
				}
			}
		}

		active[i] = !collided;
		b.x[b.x.length] = x;
		b.y[b.y.length] = y;

		if (commit) {
			addVert(bw,x,y);
			if (collided) {
				bw.active = false;
				if (collidedPeon) {
					collidedPeon.enlightened = true;
					collidedPeon.tint = bw.style;
					playHit();
				}
			}
		}

	} //for (bows)


	//check if range is still valid:
	while (ai <= bi && !active[ai]) ++ai;
	while (ai <= bi && !active[bi]) --bi;
	var valid = (ai <= bi);
	if (!valid) {
		tRange[0] = -1;
		tRange[1] = -2;
	}
	for (var i = ai; i <= bi; ++i) {
		if (!active[i]) valid = false;
	}
	if (valid) {
		tRange[0] = ai;
		tRange[1] = bi;
		return true;
	} else {
		return false;
	}
}


function draw(timestamp) {
	var elapsed = 0;
	if (lastTimestamp) {
		elapsed = (timestamp - lastTimestamp) / 1000.0;
	} else {
		lastTimestamp = timestamp;
	}
	if (elapsed < 1.0 / 60.0) {
		window.requestAnimFrame(draw);
		return;
	}

	if (elapsed > 1.0 / 10.0) {
		elapsed = 1.0 / 10.0;
	}

	lastTimestamp = timestamp;

//------------------------------------------
	if (state == STATE_LOADING) {
		if (pendingAssets == 0) {
			stateFadeOut = true;
		}
	}

	if (stateFadeOut) {
		stateFade -= elapsed;
		if (stateFade < 0.0) {
			stateFade = 0.0;
			stateFadeOut = false;
			//Transition states...
			if (state == STATE_LOADING) {
				//Welcome to Warp Zone:
				var ln = parseInt(window.location.search.substr(1));
				if (ln >= 0 && ln < levels.length) {
					levelIndex = ln;
					state = STATE_LEVEL;
					setupBoard();
				} else if (ln >= levels.length) {
					setupCrowd();
					state = STATE_OUTRO;
					troStep = 0;
				} else {
					setupCrowd();
					state = STATE_INTRO;
					troStep = 0;
				}
				thunderAudio.play();
				glimmersAudio.pause();
			} else if (state == STATE_INTRO) {
				troStep = troStep + 1;
				if (troStep < introTextures.length) {
					//great, we'll stay with this state.
				} else {
					state = STATE_LEVEL;
					setupBoard();
					thunderAudio.play();
					glimmersAudio.pause();
				}
			} else if (state == STATE_LEVEL) {
				//advance to next level (?)
				levelIndex += 1;
				if (levelIndex < levels.length) {
					setupBoard();
				} else {
					setupCrowd();
					state = STATE_OUTRO;
					troStep = 0;
					thunderAudio.pause();
					glimmersAudio.play();
					glimmersAudio.volume = 1.0;
				}
			} else if (state == STATE_OUTRO) {
				troStep = troStep + 1;
				if (troStep < outroTextures.length) {
					//great, we'll stay with this state.
				} else {
					//go back to the start, I guess(?)
					setupCrowd();
					state = STATE_INTRO;
					troStep = 0;
					levelIndex = 0;
					thunderAudio.play();
					glimmersAudio.pause();
				}
			} else {
				console.debug("Unknown state: " + state);
			}
		}
	} else {
		stateFade += elapsed;
		if (stateFade > 1.0) {
			stateFade = 1.0;
			if (state == STATE_INTRO) {
				//show onward button.
				document.getElementById("onward").hidden = false;
			} else if (state == STATE_LEVEL) {
				//show reset button.
				document.getElementById("reset").hidden = false;
			} else if (state == STATE_OUTRO) {
				//show onward button, maybe?
				//or perhaps reset?
				if (troStep + 1 < outroTextures.length) {
					document.getElementById("reset").hidden = true;
					document.getElementById("onward").hidden = false;
				} else {
					document.getElementById("reset").hidden = false;
					document.getElementById("onward").hidden = true;
				}
			}
		}
	}
	if (stateFade < 1.0) {
		document.getElementById("reset").hidden = true;
		document.getElementById("onward").hidden = true;
	}


//------------------------------------------

	if (mousedown) {
		growAcc += elapsed;
	} else {
		growAcc = 0.0;
	}

	var el = document.getElementById("game");
	var canvasWidth = el.width;
	var canvasHeight = el.height;

	gl.clearColor(0.5, 0.5, 0.5, 1.0);
	gl.clear(gl.COLOR_BUFFER_BIT); //| gl.STENCIL_BUFFER_BIT);

	gl.disable(gl.BLEND);

	gl.useProgram(solidProgram);

	setCameraUniforms();

	if (state == STATE_LEVEL && board) {
		//tiles:
		gl.bindBuffer(gl.ARRAY_BUFFER, board.vertexBuffer);
		var aVertex = gl.getAttribLocation(solidProgram, "aVertex");
		gl.enableVertexAttribArray(aVertex);
		gl.vertexAttribPointer(aVertex, 2, gl.FLOAT, false, 0, 0);
		gl.vertexAttrib4f(gl.getAttribLocation(solidProgram, "aColor"), 0.2, 0.2, 0.2, 1.0);
		gl.drawArrays(gl.TRIANGLES, 0, board.vertexCount);
		gl.disableVertexAttribArray(aVertex);


		//peons:
		var verts = [];
		var colors = [];
		var texCoords = [];
		var verts2 = [];
		var colors2 = [];
		var texCoords2 = [];
		for (var i = 0; i < board.peons.length; ++i) {
			var peon = board.peons[i];
			if (peon.enlightened) {
				peon.efade += elapsed / 0.3;
				if (peon.efade > 1.0) {
					peon.efade = 1.0;
				}
			}
			var tint = [
				(peon.tint[0] - 1.0) * peon.efade + 1.0,
				(peon.tint[1] - 1.0) * peon.efade + 1.0,
				(peon.tint[2] - 1.0) * peon.efade + 1.0,
				(peon.tint[3] - 1.0) * peon.efade + 1.0,
			];
			if (verts.length) {
				verts.push(verts[verts.length-2], verts[verts.length-1]);
				verts.push(peon.x,peon.y);
				colors.push(colors[colors.length-4], colors[colors.length-3], colors[colors.length-2], colors[colors.length-1]);
				colors.push(tint[0], tint[1], tint[2], tint[3]);
				texCoords.push(texCoords[texCoords.length-2], texCoords[texCoords.length-1]);
				texCoords.push(0.0,1.0);
			}
			verts.push(peon.x,peon.y);
			verts.push(peon.x+1,peon.y);
			verts.push(peon.x,peon.y+1);
			verts.push(peon.x+1,peon.y+1);
			colors.push(tint[0], tint[1], tint[2], tint[3]);
			colors.push(tint[0], tint[1], tint[2], tint[3]);
			colors.push(tint[0], tint[1], tint[2], tint[3]);
			colors.push(tint[0], tint[1], tint[2], tint[3]);
			texCoords.push(0.0, 1.0);
			texCoords.push(1.0, 1.0);
			texCoords.push(0.0, 0.0);
			texCoords.push(1.0, 0.0);
			if (peon.efade > 0.0) {
				if (verts2.length) {
					verts2.push(verts2[verts2.length-2], verts2[verts2.length-1]);
					verts2.push(peon.x,peon.y);
					colors2.push(colors2[colors2.length-4], colors2[colors2.length-3], colors2[colors2.length-2], colors2[colors2.length-1]);
					colors2.push(peon.tint[0], peon.tint[1], peon.tint[2], peon.tint[3]);
					texCoords2.push(texCoords2[texCoords2.length-2], texCoords2[texCoords2.length-1]);
					texCoords2.push(0.0,1.0);
				}


				verts2.push(peon.x,peon.y);
				verts2.push(peon.x+1,peon.y);
				verts2.push(peon.x,peon.y+peon.efade);
				verts2.push(peon.x+1,peon.y+peon.efade);
				colors2.push(peon.tint[0], peon.tint[1], peon.tint[2], peon.tint[3]);
				colors2.push(peon.tint[0], peon.tint[1], peon.tint[2], peon.tint[3]);
				colors2.push(peon.tint[0], peon.tint[1], peon.tint[2], peon.tint[3]);
				colors2.push(peon.tint[0], peon.tint[1], peon.tint[2], peon.tint[3]);
				texCoords2.push(0.0, 1.0);
				texCoords2.push(1.0, 1.0);
				texCoords2.push(0.0, 1.0 - peon.efade);
				texCoords2.push(1.0, 1.0 - peon.efade);
			}
		}

		gl.enable(gl.BLEND);
		gl.blendEquation(gl.FUNC_ADD);
		gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);

		gl.useProgram(texProgram);
		setCameraUniforms();

		gl.uniform1i(gl.getUniformLocation(texProgram, "uTex"), 0);

		gl.bindTexture(gl.TEXTURE_2D, peonTexture);
		glEmit(verts,colors,texCoords);
		gl.bindTexture(gl.TEXTURE_2D, null);

		if (verts2.length) {
			gl.bindTexture(gl.TEXTURE_2D, peoneTexture);
			glEmit(verts2,colors2,texCoords2);
			gl.bindTexture(gl.TEXTURE_2D, null);
		}

		gl.useProgram(solidProgram);
		setCameraUniforms();

		gl.disable(gl.BLEND);

	}


	if (state == STATE_LEVEL) {
		var clearBow = false;
		if (bow == null) {
			setupBow();
			clearBow = true;
		}
		var verts = [];
		var starts = [];
		var counts = [];
		var max_length = 1;
		for (var bi = 0; bi < bow.length; ++bi) {
			if (bow[bi].x.length > max_length) {
				max_length = bow[bi].x.length;
			}
		}
		for (var bi = 0; bi < bow.length; ++bi) {
			var b = bow[bi];
			var w = BOW_STRIPE_RADIUS;
			var f = -1.0 / max_length;
			starts.push(verts.length / 3);
			for (var i = 0; i + 1 < b.x.length; ++i) {
				var cur = {
					x: b.x[i],
					y: b.y[i]
				};
				var next = {
					x: b.x[i+1],
					y: b.y[i+1]
				};
				var perp = normalize({
					x:-(next.y - cur.y),
					y: next.x - cur.x
				});
	
				if (i == 0 && verts.length) {
					verts.push(verts[verts.length-3], verts[verts.length-2], verts[verts.length-1]);
					verts.push(cur.x - w * perp.x, cur.y - w * perp.y, i * f);
				}
	
				verts.push(cur.x - w * perp.x, cur.y - w * perp.y, i * f);
				verts.push(cur.x + w * perp.x, cur.y + w * perp.y, i * f);
	
				verts.push(next.x - w * perp.x, next.y - w * perp.y, (i+1)*f);
				verts.push(next.x + w * perp.x, next.y + w * perp.y, (i+1)*f);
			}
			counts.push(verts.length / 3 - starts[starts.length-1]);
		}
		//Actually draw:
		//gl.enable(gl.BLEND);
		//gl.blendEquation(gl.FUNC_ADD);
		//gl.enable(gl.STENCIL_TEST);
		//gl.stencilFunc(gl.NOTEQUAL, 1, 0xff);
		//gl.stencilOp(gl.KEEP, gl.REPLACE, gl.REPLACE);
	
		var vertexBuffer = gl.createBuffer();
		gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
		gl.bufferData(gl.ARRAY_BUFFER, new Float32Array( verts ), gl.STREAM_DRAW);
		var aVertex = gl.getAttribLocation(solidProgram, "aVertex");
		gl.enableVertexAttribArray(aVertex);
		gl.vertexAttribPointer(aVertex, 3, gl.FLOAT, false, 0, 0);
	
		gl.enable(gl.DEPTH_TEST);
		gl.depthFunc(gl.LESS);
	
		//glow the rainbows:
		//gl.blendFunc(gl.SRC_ALPHA, gl.ONE);
		for (var bi = 0; bi < bow.length; ++bi) {
			//gl.stencilFunc(gl.NOTEQUAL, 1 + bi, 0xff);
			var b = bow[bi];
			gl.vertexAttrib4fv(gl.getAttribLocation(solidProgram, "aColor"), b.style);
			gl.drawArrays(gl.TRIANGLE_STRIP, starts[bi], counts[bi]);
		}
		gl.disableVertexAttribArray(aVertex);
	
	
		gl.deleteBuffer(vertexBuffer);
		vertexBuffer = null;
	
		gl.disable(gl.DEPTH_TEST);
		//gl.disable(gl.STENCIL_TEST);
	
		if (clearBow) {
			bow = null;
		}
	} //if (state == LEVEL)

	var lm = localMouse();
	if (state == STATE_LEVEL && bow) {
		if ((!growing || growRange == null) && lm) {
			var close = bow.length;
			var close_dis = 100.0 * 100.0;
			for (var i = 0; i < bow.length; ++i) {
				var b = bow[i];
				if (!b.active) continue;
				var at = {
					x:b.x[b.x.length-1],
					y:b.y[b.y.length-1]
				};
				var dis = (lm.x - at.x) * (lm.x - at.x) + (lm.y - at.y) * (lm.y - at.y);
				if (dis < close_dis) {
					close = i;
					close_dis = dis;
				}
			}
			if (close < bow.length) {
				//find active segment based on the close length:
				var ai = close;
				var bi = close;
				while (ai - 1 >= 0 && bow[ai-1].active) --ai;
				while (bi + 1 < bow.length && bow[bi+1].active) ++bi;

				growRange = [ai, bi];
			} else {
				growRange = null;
			}
		}


		(function() {//Draw bow ends for non-growing bows:
			var verts = [];
			var colors = [];
			var sVerts = [];
			var sColors = [];
			var sTexCoords = [];
			for (var i = 0; i < bow.length; ++i) {
				var b = bow[i];
				if (!b.active) continue;
				if (growRange && i >= growRange[0] && i <= growRange[1]) {
					b.shift = (b.shift + 0.3 * elapsed) % 1.0;
					continue;
				} else {
						b.shift = (b.shift + 0.1 * elapsed) % 1.0;
				}
				var at = {
					x:b.x[b.x.length-1],
					y:b.y[b.y.length-1],
				};
				var norm = normalize({
					x:at.x - b.x[b.x.length-2],
					y:at.y - b.y[b.y.length-2],
				});
				var perp = {
					x:BOW_STRIPE_RADIUS * norm.y,
					y:BOW_STRIPE_RADIUS *-norm.x,
				};
				if (verts.length) {
					verts.push(verts[verts.length-2], verts[verts.length-1]);
					verts.push(at.x-perp.x,at.y-perp.y);
					colors.push(colors[colors.length-4], colors[colors.length-3], colors[colors.length-2], colors[colors.length-1]);
					colors.push(b.style[0], b.style[1], b.style[2], b.style[3]);
				}

				var l0 = 1.5;
				verts.push(at.x-perp.x,at.y-perp.y);
				verts.push(at.x+perp.x,at.y+perp.y);
				verts.push(at.x-perp.x+l0*norm.x,at.y-perp.y+l0*norm.y);
				verts.push(at.x+perp.x+l0*norm.x,at.y+perp.y+l0*norm.y);
				colors.push(b.style[0], b.style[1], b.style[2], 0.5 * b.style[3]);
				colors.push(b.style[0], b.style[1], b.style[2], 0.5 * b.style[3]);
				colors.push(b.style[0], b.style[1], b.style[2], 0.0);
				colors.push(b.style[0], b.style[1], b.style[2], 0.0);

				var w = 0.1
				var x0 = b.bias;
				var x1 = b.bias + w;
				var p =-b.shift;

				if (sVerts.length) {
					sVerts.push(sVerts[sVerts.length-2], sVerts[sVerts.length-1]);
					sVerts.push(at.x-perp.x,at.y-perp.y);
					sColors.push(sColors[sColors.length-4], sColors[sColors.length-3], sColors[sColors.length-2], sColors[sColors.length-1]);
					sTexCoords.push(sTexCoords[sTexCoords.length-2], sTexCoords[sTexCoords.length-1]);
					sColors.push(1.0,1.0,1.0,0.5);
					sTexCoords.push(x0,p);
				}

				var l1 = 1.0;
				var l2 = 2.0;
				sVerts.push(at.x-perp.x,at.y-perp.y);
				sVerts.push(at.x+perp.x,at.y+perp.y);
				sVerts.push(at.x-perp.x+l1*norm.x,at.y-perp.y+l1*norm.y);
				sVerts.push(at.x+perp.x+l1*norm.x,at.y+perp.y+l1*norm.y);
				sVerts.push(at.x-perp.x+l2*norm.x,at.y-perp.y+l2*norm.y);
				sVerts.push(at.x+perp.x+l2*norm.x,at.y+perp.y+l2*norm.y);
				sColors.push(1.0,1.0,1.0,0.5);
				sColors.push(1.0,1.0,1.0,0.5);
				sColors.push(1.0,1.0,1.0,1.0);
				sColors.push(1.0,1.0,1.0,1.0);
				sColors.push(1.0,1.0,1.0,0.0);
				sColors.push(1.0,1.0,1.0,0.0);

				sTexCoords.push(x0,p);
				sTexCoords.push(x1,p);
				p += w * l1 / (2.0 * BOW_STRIPE_RADIUS) * 0.25;
				sTexCoords.push(x0,p);
				sTexCoords.push(x1,p);
				p += w * 0.8 * (l2-l1) / (2.0 * BOW_STRIPE_RADIUS) * 0.25;
				sTexCoords.push(x0,p);
				sTexCoords.push(x1,p);
			}
			gl.enable(gl.BLEND);
			gl.blendEquation(gl.FUNC_ADD);
			gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
			glEmit(verts,colors);

			gl.useProgram(texProgram);
			setCameraUniforms();
			gl.uniform1i(gl.getUniformLocation(texProgram, "uTex"), 0);

			gl.bindTexture(gl.TEXTURE_2D, shimmerTexture);

			glEmit(sVerts,sColors,sTexCoords);

			gl.bindTexture(gl.TEXTURE_2D, null);

			gl.useProgram(solidProgram);
			setCameraUniforms();

			gl.disable(gl.BLEND);
		})();


		if (growRange && lm) {
			var growSteps = 0;
			if (growing) {
				while (growAcc > TICK) {
					growAcc -= TICK;
					growSteps += 1;
				}
			}

			var tRange = [growRange[0], growRange[1]];
			var tx = [];
			var ty = [];
			var sP = [];

			for (var i = 0; i < bow.length; ++i) {
				tx[i] = bow[i].x.slice(-2);
				ty[i] = bow[i].y.slice(-2);
				sP[i] = -bow[i].shift;
			}

			var verts = [];
			var colors = [];
			var sVerts = [];
			var sColors = [];
			var sTexCoords = [];

			for (var t = 0; t < 20; ++t) {
				var tfade = t / 19.0;
				var commit = (t < growSteps);
				if (growStep(lm, tRange, tx,ty,sP, tfade,verts,colors,sVerts,sColors,sTexCoords, commit)) {
					if (commit) {
						growRange = [tRange[0], tRange[1]];
					}
				} else {
					if (commit) {
						growRange = null;
						if (tRange[0] == -1) {
							growing = false;
						}
					}
					break;
				}
			}

			if (verts.length) {
				gl.enable(gl.BLEND);
				gl.blendEquation(gl.FUNC_ADD);
				gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
				glEmit(verts,colors);

				gl.useProgram(texProgram);
				setCameraUniforms();
				gl.uniform1i(gl.getUniformLocation(texProgram, "uTex"), 0);

				gl.bindTexture(gl.TEXTURE_2D, shimmerTexture);

				glEmit(sVerts,sColors,sTexCoords);

				gl.bindTexture(gl.TEXTURE_2D, null);

				gl.useProgram(solidProgram);
				setCameraUniforms();

				gl.disable(gl.BLEND);
			}


		}


	} //end if(bow)

	//check win condition:
	if (state == STATE_LEVEL && board) {
		var enlightened = 0;
		for (var i = 0; i < board.peons.length; ++i) {
			if (board.peons[i].enlightened) {
				++enlightened;
			}
		}
		if (enlightened == board.peons.length) {
			var ow = document.getElementById("onward");
			ow.hidden = false;
		}
		var vol = (enlightened / board.peons.length) * 0.75;
		if (growing && growRange) vol += 0.25;
		glimmersAudio.volume = vol;
		if (vol > 0.0) {
			if (glimmersAudio.paused) {
				glimmersAudio.play();
			}
		} else {
			if (!glimmersAudio.paused) {
				glimmersAudio.pause();
			}
		}
	}

/*
	if (lm) {
		var verts = [
			lm.x - 1.0, lm.y - 0.5,
			lm.x + 1.0, lm.y + 0.5,
			lm.x - 1.0, lm.y + 0.5,
			lm.x + 1.0, lm.y - 0.5,
		];
		var vertexBuffer = gl.createBuffer();
		gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
		gl.bufferData(gl.ARRAY_BUFFER, new Float32Array( verts ), gl.STREAM_DRAW);
		var attrib = gl.getAttribLocation(solidProgram, "aVertex");
		gl.enableVertexAttribArray(attrib);
		gl.vertexAttribPointer(attrib, 2, gl.FLOAT, false, 0, 0);
		gl.drawArrays(gl.LINES, 0, verts.length / 2);
		gl.disableVertexAttribArray(attrib);

		gl.deleteBuffer(vertexBuffer);
	}
*/

	setScreenUniforms();

	if (state == STATE_LOADING) {
		var verts = [];
		var colors = [];
		//TODO....
	} else if (state == STATE_INTRO || state == STATE_OUTRO) {
		gl.useProgram(texProgram);
		setScreenUniforms();

		var a = canvasWidth / canvasHeight;
		var verts = [
			-a,-1.0,
			 a,-1.0,
			-a, 1.0,
			 a, 1.0,
		];
		var texCoords = [
			0.0, 1.0,
			1.0, 1.0,
			0.0, 0.0,
			1.0, 0.0,
		];

		if (state == STATE_INTRO) {
			gl.bindTexture(gl.TEXTURE_2D, introTextures[troStep]);
		} else {
			gl.bindTexture(gl.TEXTURE_2D, outroTextures[troStep]);
		}

		gl.uniform1i(gl.getUniformLocation(texProgram, "uTex"), 0);

		glEmit(verts,[1.0, 1.0, 1.0, 1.0],texCoords);

		verts = [];
		var colors = [];
		texCoords = [];

		for (var i = 0; i < crowd.length; ++i) {
			var c = crowd[i];
			c.acc = (c.acc + elapsed * 0.3) % c.period;

			var atX = c.x;
			var atY = c.y;
			var rightX = 1.0;
			var rightY = 0.0;

			if (state == STATE_OUTRO) {
				if (c.acc < 0.1) {
					var amt = c.acc / 0.1;
					if (amt < 0.6) {
						var x = amt / 0.6;
						atY += x - x * x;
					} else if (amt < 0.9) {
						var x = (amt - 0.6) / 0.3;
						atY += (x - x * x) * 0.5;
					} /*else {
						var x = (amt - 0.85) / 0.15;
						atY += (x - x * x) * 0.25;
					}*/

				}
			} else {
				//they waggle in the intro.
				if (c.acc < 0.2) {
					var amt = c.acc / 0.2;
					var ang = Math.sin(amt * 2.0 * Math.PI) * 0.1;
					rightX = Math.cos(ang);
					rightY = Math.sin(ang);
				}
			}


			var upX =-rightY;
			var upY = rightX;

			verts.push(atX - CROWD_RAD * rightX, atY - CROWD_RAD * rightY);
			verts.push(atX + CROWD_RAD * rightX, atY + CROWD_RAD * rightY);
			verts.push(atX + CROWD_RAD * rightX + 2.0 * CROWD_RAD * upX, atY + CROWD_RAD * rightY + 2.0 * CROWD_RAD * upY);

			verts.push(atX - CROWD_RAD * rightX, atY - CROWD_RAD * rightY);
			verts.push(atX + CROWD_RAD * rightX + 2.0 * CROWD_RAD * upX, atY + CROWD_RAD * rightY + 2.0 * CROWD_RAD * upY);
			verts.push(atX - CROWD_RAD * rightX + 2.0 * CROWD_RAD * upX, atY - CROWD_RAD * rightY + 2.0 * CROWD_RAD * upY);

			var tint = c.tint.slice(0);

			if (state == STATE_INTRO) {
				var amt = (tint[0] + tint[1] + tint[2]) / 3.0;
				amt = 0.5 + amt;
				tint[0] = tint[1] = tint[2] = amt;
				tint[3] = 1.0;
			}

			colors.push(tint[0], tint[1], tint[2], tint[3]);
			colors.push(tint[0], tint[1], tint[2], tint[3]);
			colors.push(tint[0], tint[1], tint[2], tint[3]);
			colors.push(tint[0], tint[1], tint[2], tint[3]);
			colors.push(tint[0], tint[1], tint[2], tint[3]);
			colors.push(tint[0], tint[1], tint[2], tint[3]);

			texCoords.push(0.0, 1.0);
			texCoords.push(1.0, 1.0);
			texCoords.push(1.0, 0.0);

			texCoords.push(0.0, 1.0);
			texCoords.push(1.0, 0.0);
			texCoords.push(0.0, 0.0);
		}


		if (state == STATE_OUTRO && troStep == 0 || state == STATE_INTRO && troStep == 2) {
			gl.enable(gl.BLEND);
			gl.blendEquation(gl.FUNC_ADD);
			gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);

			if (state == STATE_OUTRO) {
				gl.bindTexture(gl.TEXTURE_2D, peoneTexture);
			} else {
				gl.bindTexture(gl.TEXTURE_2D, peonTexture);
			}
	
			glEmit(verts,colors,texCoords,gl.TRIANGLES);


			gl.disable(gl.BLEND);
		}

		gl.bindTexture(gl.TEXTURE_2D, null);


		gl.useProgram(solidProgram);
		setScreenUniforms();
	}

	if (stateFade < 1.0) {
		gl.enable(gl.BLEND);
		gl.blendEquation(gl.FUNC_ADD);
		gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
		var verts = [
			-2.0,-2.0,
			 2.0,-2.0,
			-2.0, 2.0,
			 2.0, 2.0,
		];
		var vertexBuffer = gl.createBuffer();
		gl.bindBuffer(gl.ARRAY_BUFFER, vertexBuffer);
		gl.bufferData(gl.ARRAY_BUFFER, new Float32Array( verts ), gl.STREAM_DRAW);
		var aVertex = gl.getAttribLocation(solidProgram, "aVertex");
		gl.enableVertexAttribArray(aVertex);
		gl.vertexAttribPointer(aVertex, 2, gl.FLOAT, false, 0, 0);
		gl.vertexAttrib4f(gl.getAttribLocation(solidProgram, "aColor"), 0.0, 0.0, 0.0, 1.0 - stateFade);
		gl.drawArrays(gl.TRIANGLE_STRIP, 0, verts.length / 2);
		gl.disableVertexAttribArray(aVertex);
		gl.deleteBuffer(vertexBuffer);

		gl.disable(gl.BLEND);
	}

	window.requestAnimFrame(draw);
}

function reset(evt) {
	if (state == STATE_LEVEL) {
		setupBoard();
	} else if (state == STATE_OUTRO) {
		stateFadeOut = true;
	}
	evt.preventDefault();
	return false;
}

function onward(evt) {
	if (state == STATE_LEVEL) {
		if (board) {
			var good = true;
			for (var i = 0; i < board.peons.length; ++i) {
				if (!board.peons[i].enlightened) {
					good = false;
				}
			}
			if (good) {
				stateFadeOut = true;
			}
		}
	} else if (state == STATE_INTRO || state == STATE_OUTRO) {
		stateFadeOut = true;
	}
	evt.preventDefault();
	return false;
}


function setup() {

	

	var el = document.getElementById("game");

	initWebGL(el);

	if (!gl) {
		return;
	}

	solidProgram = getProgram("solid-fs","solid-vs");

	if (!solidProgram) {
		return;
	}

	texProgram = getProgram("tex-fs","tex-vs");

	if (!texProgram) {
		return;
	}


	//After: http://www.html5canvastutorials.com/advanced/html5-canvas-mouse-coordinates/
	window.addEventListener('mousemove', function(evt) {
		var rect = el.getBoundingClientRect();
		mouse = {
			x: evt.clientX - rect.left,
			y: evt.clientY - rect.top,
			};
		var lm = localMouse();
		if (lm) {
			bowStart.x = lm.x;
		}
	});
	el.addEventListener('mousedown', function(evt) {
		if (state == STATE_LEVEL) {
			if (!bow) {
				setupBow();
				playGliss();
			} else {
				mousedown = true;
				growing = true;
			}
		}
		//remove dragging (?)
		evt.preventDefault();
		return false;
	});
	window.addEventListener('mouseup', function(evt) {
		mousedown = false;
		growing = false;
	});

	var rs = document.getElementById("reset");
	rs.addEventListener('mousedown', reset);
	rs.hidden = true;

	var ow = document.getElementById("onward");
	ow.addEventListener('mousedown', onward);
	ow.hidden = true;

	var mute = document.getElementById("mute");
	var unmute = document.getElementById("unmute");
	mute.addEventListener('mousedown', function(evt) {
		setMuted(true);
		mute.hidden = true;
		unmute.hidden = false;
		evt.preventDefault();
		return false;
	});

	unmute.addEventListener('mousedown', function(evt) {
		setMuted(false);
		mute.hidden = false;
		unmute.hidden = true;
		evt.preventDefault();
		return false;
	});
	unmute.hidden = true;

	function setupLoad(asset, src, callback) {
		++pendingAssets;
		++totalAssets;
		console.debug("Pending: " + pendingAssets + " of " + totalAssets);

		var loadInfo = [ 0 ];
		function loaded() {
			callback(asset);
			if (loadInfo[0] == 0) {
				--pendingAssets;
				console.debug("Pending: " + pendingAssets + " of " + totalAssets);
				if (pendingAssets <= 0) {
					stateFadeOut = true;
				}
				loadInfo[0] = 1;
			}
		}
		if (asset.play) {
			asset.addEventListener('canplaythrough', loaded);
		} else {
			asset.onload = loaded;
		}
		asset.onerror = function() {
			alert("Failed to load " + src);
		}
		asset.src = src;
	}
	setupLoad(new Image(), intro0TexData, function(img) {
		introTextures[0] = textureFromImage(img);
	});
	setupLoad(new Image(), intro1TexData, function(img) {
		introTextures[1] = textureFromImage(img);
	});
	setupLoad(new Image(), intro2TexData, function(img) {
		introTextures[2] = textureFromImage(img);
	});
	setupLoad(new Image(), outro0TexData, function(img) {
		outroTextures[0] = textureFromImage(img);
	});
	setupLoad(new Image(), outro1TexData, function(img) {
		outroTextures[1] = textureFromImage(img);
	});
	setupLoad(new Image(), peonTexData, function(img) {
		peonTexture = textureFromImage(img, true);
	});
	setupLoad(new Image(), peoneTexData, function(img) {
		peoneTexture = textureFromImage(img, true);
	});
	setupLoad(new Image(), shimmerTexData, function(img) {
		shimmerTexture = textureFromImage(img);
		gl.bindTexture(gl.TEXTURE_2D, shimmerTexture);
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S, gl.REPEAT);
		gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T, gl.REPEAT);
		gl.bindTexture(gl.TEXTURE_2D, null);

	});

	setupLoad(new Audio(), "thunder-loop.ogg", function(aud) {
		thunderAudio = aud;
		thunderAudio.loop = true;
	});
	setupLoad(new Audio(), "glimmers-loop.ogg", function(aud) {
		glimmersAudio = aud;
		glimmersAudio.loop = true;
	});
	setupLoad(new Audio(), "gliss.ogg", function(aud) {
		glissAudio = aud;
	});
	setupLoad(new Audio(), "hit1.ogg", function(aud) {
		hitsAudio[0] = aud;
	});
	setupLoad(new Audio(), "hit2.ogg", function(aud) {
		hitsAudio[1] = aud;
	});
	setupLoad(new Audio(), "hit3.ogg", function(aud) {
		hitsAudio[2] = aud;
	});
	setupLoad(new Audio(), "hit4.ogg", function(aud) {
		hitsAudio[3] = aud;
	});
	setupLoad(new Audio(), "hit5.ogg", function(aud) {
		hitsAudio[4] = aud;
	});


	window.requestAnimFrame =
		window.requestAnimationFrame
		|| window.webkitRequestAnimationFrame
		|| window.mozRequestAnimationFrame
		|| window.oRequestAnimationFrame
		|| window.msRequestAnimationFrame
		|| function(callback) {
          window.setTimeout(callback, 1000 / 60);
        };
	window.requestAnimFrame(draw);


}
